#include <iostream>
#include <fstream>
#include <cstdlib>
#include <cstring>
#include <string>
#include "Course.h"

using std::cerr;
using std::cout;
using std::endl;
using std::ifstream;
using std::string;

// Course constructor

Course::Course(const char* new_course_number, const char* new_name)
{
    strcpy(name, new_name);
    strcpy(course_number, new_course_number);
    num_students = 0;
    
}

// Other member functions

void Course::read_student_data(const string& file_name)
{
    ifstream in_file;
    char first_name[11], last_name[21], name[31];
    double gpa;
    
    // Open the input file and test for failure
    in_file.open(file_name);
    if (!in_file)
    {
        cerr << "Error - unable to open input file " << file_name << endl;
        exit(1);
    }
    
    in_file >> first_name;
    while (in_file)
    {
        in_file >> last_name;
        in_file >> gpa;
        
        strcpy(name, last_name);
        strcat(name, ", ");
        strcat(name, first_name);

        // Create a Student object and copy it into
        // the array.
        class_list[num_students] = Student(name, gpa);
        
        num_students++;
        
        in_file >> first_name;
    }
    
    in_file.close();
}

void Course::sort_class_list(bool (*compare)(const Student&, const Student&))
{
    int i, j, min;
    Student temp;
    
    for (i = 0; i < num_students - 1; i++)
    {
        min = i;
        
        for (j = i + 1; j < num_students; j++)
        {
            if (compare(class_list[j], class_list[min]))
                min = j;
        }
        
        temp = class_list[i];
        class_list[i] = class_list[min];
        class_list[min] = temp;
    }
}

void Course::print() const
{
    int i;
    
    cout << course_number << " " << name << endl << endl;
    
    for (i = 0; i < num_students; i++)
    {
        class_list[i].print();
        cout << endl;
    }
}

// Used to sort in ascending order by name
bool name_less(const Student& s1, const Student& s2)
{
    return (strcmp(s1.get_name(), s2.get_name()) < 0);
}

// Used to sort in descending order by name
bool name_greater(const Student& s1, const Student& s2)
{
    return (strcmp(s1.get_name(), s2.get_name()) > 0);
}

// Used to sort in ascending order by gpa
bool gpa_less(const Student& s1, const Student& s2)
{
    return (s1.get_gpa() < s2.get_gpa());
}

// Used to sort in descending order by gpa
bool gpa_greater(const Student& s1, const Student& s2)
{
    return (s1.get_gpa() > s2.get_gpa());
}
